// Scrollbar
// a scrollbar/slider widget
// 2000.03.13

// DynAPI Copyright (C) 2000 Dan Steinman
// Modified by IlMaestro ( ilmaestro@cantir.com )
// Redistributable under the terms of the GNU Library General Public License
// Available at http://www.dansteinman.com/dynapi/

function ScrollBar(type) {
	this.component = new DynLayer()
	this.component.setParentComponent(this)
	this.knob = this.component.addChild(new DynLayer())

	this.dragEvents = new EventListener(this)
	this.dragEvents.ondragmove = function (e) {
		var o = e.getTarget()
		o.findRatio()
		o.invokeEvent("scroll")
		e.setBubble(false)
	}
	this.dragEvents.ondragstart = function (e) {
		var o = e.getTarget()
		e.setBubble(false)
	}
	this.dragEvents.ondragend = function (e) {
		var o = e.getTarget()
		e.setBubble(false)
	}
	DragEvent.setDragBoundary(this.knob)
	DragEvent.enableDragEvents(this.knob)
	this.knob.addEventListener(this.dragEvents)
		
	this.mouseEvents = new EventListener(this)
	this.mouseEvents.onmousemove = function(e) {
		if (e.getTarget().knob.scrollActive) e.setBubble(false)
	}
	this.mouseEvents.onmousedown = function(e) {
		var lyr = e.getSource()
		var o = e.getTarget()
		if (!lyr.slideActive) {
			var newx = e.getX()-o.knob.w/2
			var newy = e.getY()-o.knob.h/2
			var offW = o.getWidth()-o.knob.w
			var offH = o.getHeight()-o.knob.h
			if (newx<0) newx=0
			if (newx>=offW) newx=offW
			if (newy<0) newy=0
			if (newy>=offH) newy=offH
			o.knob.slideTo(newx,newy)
		}
		e.setBubble(false)
	}
	this.mouseEvents.onmouseup = function(e) {
		e.getTarget().knob.slideActive = false
	}
	this.component.addEventListener(this.mouseEvents)
	
	this.slideEvents = new EventListener(this)
	this.slideEvents.onslide = function(e) {
		var o = e.getTarget()
		o.findRatio()
		o.invokeEvent("scroll")
	}
	this.slideEvents.onslideend = function(e) {
		var lyr = e.getSource()
		var o = e.getTarget()
		var evt = new MouseEvent()
		evt.bubble = false
		evt.type = "mousedown"		
		evt.src = lyr
		evt.x = o.knob.getWidth()/2
		evt.y = o.knob.getHeight()/2
		evt.pageX = o.knob.getPageX()+evt.x
		evt.pageY = o.knob.getPageY()+evt.y
		lyr.invokeEvent("mousedown",evt)
	}
	this.knob.addEventListener(this.slideEvents)
	
	if (type!=null) this.setOrientation(type)
}

WidgetInterface.inherit(ScrollBar)

ScrollBar.prototype.ratiox = 0
ScrollBar.prototype.ratioy = 0

ScrollBar.prototype.setSize = function(w,h) {
	if (w==null) w = this.getWidth()
	if (h==null) h = this.getHeight()
	this.component.setSize(w,h)
	if (this.knob.x+this.knob.w>this.getWidth()) this.knob.setX(this.getWidth()-this.knob.w)
	if (this.knob.y+this.knob.h>this.getHeight()) this.knob.setY(this.getHeight()-this.knob.h)
	this.findRatio()
}
ScrollBar.prototype.setWidth = function(w) {
	this.setSize(w,null)
}
ScrollBar.prototype.setHeight = function(h) {
	this.setSize(null,h)
}
ScrollBar.prototype.setOrientation = function(type) { // 0=horz,1=vert
	this.horizontal = !type
	this.vertical = type
}
ScrollBar.prototype.setImages = function(troughImage,knobImage) {
	if (troughImage) {
		this.troughImage = troughImage
		this.component.setBgImage(this.troughImage.src)
		if (this.vertical) this.setWidth(this.troughImage.w)
		if (this.horizontal) this.setHeight(this.troughImage.h)
	}
	if (knobImage) {
		this.knobImage = knobImage
		this.knob.setBgImage(this.knobImage.src)
		this.knob.setSize(this.knobImage.w,this.knobImage.h)
	}
}
ScrollBar.prototype.setRatioX = function(rx) {this.setRatio(rx,0)}
ScrollBar.prototype.setRatioY = function(ry) {this.setRatio(0,ry)}
ScrollBar.prototype.setRatio = function(rx,ry) {
	this.ratiox=rx
	this.ratioy=ry
	this.knob.moveTo(this.ratiox*(this.w-(this.dw)),this.ratioy*(this.h-(this.dh)))
}
ScrollBar.prototype.findRatio = function() {
	var tx = (this.getWidth()-this.knob.w)
	var ty = (this.getHeight()-this.knob.h)
	this.ratiox = tx==0 ? 0 : (this.knob.x)/tx
	this.ratioy = ty==0 ? 0 : (this.knob.y)/ty
}
ScrollBar.prototype.getRatioX = function() {return this.ratiox}
ScrollBar.prototype.getRatioY = function() {return this.ratioy}
ScrollBar.prototype.reset = function() { this.knob.moveTo(0,0); this.ratiox=this.ratioy=0; }
