// Tree and TreeNode
// a generic tree selection widget
// 2000.05.03
//
// Copyright (C) 2000 Brandon E. Myers
// Redistributable under the terms of the GNU Library General Public License Version 2
// www.gnu.org

// This code utilizes the core DynAPI v2
// Copyright (C) 2000 Dan Steinman
// Redistributable under the terms of the GNU Library General Public License
if (!WidgetInterface) DynAPI.loadSCRIPT("../js/gui/widget.js");
if (!Label) DynAPI.loadSCRIPT("../js/gui/label.js");

function Tree(rootText,imgSrc,hideRoot) {
	this.treeImages=imgSrc; // default dir for tree images is specified in the first param.
	this.imagePrefix=""; // Prefix added before the standard image names.
	this.linePool=[];    // Pool of unused TreeLine objects.
	this.usedLines=[];   // List of TreeLine objects which are currently in use.
	this.imagePool=[];   // Pool of images so that duplicates are not loaded. (DynAPI should implement this)
	this.masterIndex=[];
	this.maxWidth=0;  // Used to prevent the tree from being larger that a certian width.
	this.maxHeight=0; // Used to prevent the tree from being larger that a certian height.
	// If the actual width/height is larger than the max width/height, a sizeexception event is invoked.
	// This allows an event to capture and enable and control scrollbars and scrolling.
	this.hideRoot=(hideRoot?true:false);
	this.horizScroll=false; // Horizontal scrollbars are visible.
	this.vertScroll=false;  // Vertical scrollbars are visible.
	this.enableHoriz=false; // allow horizontal scroll bars.
	this.enableVert=true;   // allow vertical scroll bars.
	this.scrollInner=true;  // If true, the scroll bars are added within the maxWidth/maxHeight bounds.
	                        // If false, the scroll bars are added outside of the max size boundries.
	this.preferredWidth=0;  // If the actual width is greater and enableHoriz==true, scrollbars are added.
	this.preferredHeight=0; // If the actual height is greater and enableVert==true, scrollbars are added.
	this.hoverPeriod=250;   // time in MS to activate hovering when mouse-over event occurs.
	this.showFullNameOnHover=false;  // If the tree item's renderObject is obstructed from full view,
	                                 // and the mouse is hovering, and this value is true, the 
	                                 // TreeNode's getText method called and the value is shown above all other objects.
    this.showJoinLines=true;  // show the vertical lines joining siblings when children seperate the siblings.
    this.multiSelect = false; // Allow selecting of multiple tree nodes at one time. 
                              // Only leaf items are allowed to be multi-selected.
	this.component = new DynLayer(); // The top level layer for the entire tree.
	this.component.target = this; // I believe the target is the target for events.
	this.component.onCreate = function() {
		var o = this.target;
		o.maxWidth = 0;
		o.totalHeight = 0;
		o.root.moveTo(0,0);
		var dw = o.root.getWidth();
		var dh = o.root.getHeight();
		if (!dw) {
			for (i in o.root.children) {
				var item = o.root.children[i];
        		if (o.hideRoot)
        		    item.setX(0);
				//dw += item.getComponent().w;
				dh += item.getComponent().h;
				//if (w>o.maxWidth) o.maxWidth=w;
			}
			//dw = o.maxWidth;
		}
		o.setWidth(dw);
		o.setHeight(dh+1);
		//o.setPreferredWidth(dw);
		if (o.hideRoot) o.root.expand();
	}

    this.hoverLayer=new DynLayer();
    this.hoverLayer.setVisible(false);
    DynAPI.getDocument().addChild(this.hoverLayer);
    //this.setVisible("hidden");
    this.initImages();
	this.root = new TreeNode(rootText); // The root node to which all top level nodes are added.
	this.root.tree=this; // Set the root node's tree reference.
	this.root.parent=this.root;
	this.root.setRelations(); // fixes images as well.
	this.addChild(this.root); // Add the root node to the visual buffer.
	//this.setBgColor("yellow");
}

/**
  * Adds a node to the master index of all nodes.
  */
//Tree.prototype.addNode=function (node) {
Tree.prototype.addNode=function(node,f) {
     if (f) {// we where called by TreeNode.. add to master index list.
        if (node.masterIndex==-1) { // don't add to master index if it is already in it.
            node.masterIndex=this.masterIndex.length;
            this.masterIndex[node.masterIndex]=node;
        }
     } else {
        if (node.masterIndex!=0) this.root.append(node); // append if node is not already a child of the root.
     }
}

Tree.prototype.getImage=function(img,forceNew,height,width) {
    with(this) {
        if (!imagePool[img] || forceNew)
            imagePool[img]=DynAPI.getImage(treeImages+"/"+tree.imagePrefix+img,width,height);
        return imagePool[img];
    }
}

Tree.prototype.getLine=function() {
    var line;
        if (this.linePool.length==0) {
            line=new TreeLine(this.root);
            this.linePool[this.linePool.length]=line;
        }
        line=this.linePool[this.linePool.length-1];
        this.linePool=this.linePool.slice(0,this.linePool.length-1);
//        this.linePool.splice(this.linePool.length-1);
        line.index=this.usedLines.length;
        this.usedLines[this.usedLines.length]=line;
    return line;
}

Tree.prototype.releaseLine=function(line) {
    this.linePool[this.linePool.length]=line;
    this.usedLines=(new Array()).concat(this.usedLines.slice(0,line.index),this.usedLines.slice(line.index+1));
}

Tree.prototype.setImgDir=function(dir,reload) {
    this.treeImages=dir;
    if (reload) this.root.setRelations(); //update images and relationships.
}

    Tree.prototype.setImageDir=function(prefix,reload) {
    tree.imagePrefix=prefix;
    if(reload) this.root.setRelations();
}

/** Get the root tree node for the entire tree. */
Tree.prototype.getRoot=function() {
    return this.root;
}

/** Disassociate node from this tree */
Tree.prototype.remove=function(node) {
    if (node.masterIndex<0 || node.parent!=null) return;
    var mIdx=node.masterIndex;
    node.masterIndex=-1;
    node.tree=null;
    with (this) {
        masterIndex=(new Array()).concat(masterIndex.slice(0,mIdx),masterIndex.slice(mIdx+1));
        for(var i=mIdx;i<masterIndex.length;i++)
            masterIndex[i].masterIndex=i;
    }
}

/** This tree is now a widget */
WidgetInterface.inherit(Tree);

/**
  * The following are default styles applied to the tree model.
  */


Tree.prototype.style = new Object();
Tree.prototype.style.bgColor = '';
Tree.prototype.style.bgImage = '../images/bgTextureBlue.jpg';
Tree.prototype.style.bgColorRoll = '#eaebff';
Tree.prototype.style.bgColorSelect = '#cdceff';
Tree.prototype.style.textColor = '#000000';
Tree.prototype.style.textColorRoll = '#000000';
Tree.prototype.style.textColorSelect = '#000000';
//Tree.prototype.style.padding = 3;
Tree.prototype.nullNode = {};

/**
  * Currently selected items.
  */
Tree.prototype.selectedItem = [];

// Image names used after src dir and image prefix.
// Theming stuff...
Tree.prototype.images=new Object();;

// Node Activators
Tree.prototype.nodeTypes=[
"topNode",
"node",
"lastNode",
"blank"
];

// Icons
Tree.prototype.iconTypes=[
"parentNode",
"emptyNode",
];

for(i in Tree.prototype.nodeTypes) {
    Tree.prototype.images[Tree.prototype.nodeTypes[i]+"Open"]=new Object();
    Tree.prototype.images[Tree.prototype.nodeTypes[i]+"Close"]=new Object();
    Tree.prototype.images[Tree.prototype.nodeTypes[i]]=new Object();
}

for(i in Tree.prototype.iconTypes) {
    Tree.prototype.images[Tree.prototype.iconTypes[i]+"Open"]=new Object();
    Tree.prototype.images[Tree.prototype.iconTypes[i]+"Close"]=new Object();
}
Tree.prototype.images.line={};

// Set default values.
// Node Activators (Images)
Tree.prototype.images.topNodeOpen.src="mtopnode.gif";
Tree.prototype.images.topNodeClose.src="ptopnode.gif";
Tree.prototype.images.nodeOpen.src="mnode.gif";
Tree.prototype.images.nodeClose.src="pnode.gif";
Tree.prototype.images.node.src="node.gif";
Tree.prototype.images.lastNodeOpen.src="mlastnode.gif";
Tree.prototype.images.lastNodeClose.src="plastnode.gif";
Tree.prototype.images.lastNode.src="lastnode.gif";
Tree.prototype.images.line.src="vertline.gif";
Tree.prototype.images.blankOpen.src="blank.gif";
Tree.prototype.images.blankClose.src="blank.gif";
// Icons (Images)
Tree.prototype.images.parentNodeOpen.src="folderopen.gif";
Tree.prototype.images.parentNodeClose.src="folderclosed.gif";
Tree.prototype.images.emptyNodeOpen.src="doc.gif";
Tree.prototype.images.emptyNodeClose.src="doc.gif";

// Node Activators (Width)
Tree.prototype.images.topNodeOpen.w=16;
Tree.prototype.images.topNodeClose.w=16;
Tree.prototype.images.nodeOpen.w=16;
Tree.prototype.images.nodeClose.w=16;
Tree.prototype.images.node.w=16;
Tree.prototype.images.lastNodeOpen.w=16;
Tree.prototype.images.lastNodeClose.w=16;
Tree.prototype.images.lastNode.w=16;
Tree.prototype.images.line.w=16;
Tree.prototype.images.blankOpen.w=16;
Tree.prototype.images.blankClose.w=16;
// Icons (Width)
Tree.prototype.images.parentNodeOpen.w=16;
Tree.prototype.images.parentNodeClose.w=16;
Tree.prototype.images.emptyNodeOpen.w=16;
Tree.prototype.images.emptyNodeClose.w=16;

// Node Activators (Height)
Tree.prototype.images.topNodeOpen.h=22;
Tree.prototype.images.topNodeClose.h=22;
Tree.prototype.images.nodeOpen.h=22;
Tree.prototype.images.nodeClose.h=22;
Tree.prototype.images.node.h=22;
Tree.prototype.images.lastNodeOpen.h=22;
Tree.prototype.images.lastNodeClose.h=22;
Tree.prototype.images.lastNode.h=22;
Tree.prototype.images.line.h=22;
Tree.prototype.images.blankOpen.h=22;
Tree.prototype.images.blankClose.h=22;
// Icons (Height)
Tree.prototype.images.parentNodeOpen.h=22;
Tree.prototype.images.parentNodeClose.h=22;
Tree.prototype.images.emptyNodeOpen.h=22;
Tree.prototype.images.emptyNodeClose.h=22;

Tree.prototype.initImages=function() {
    for (var i in this.images) {
        if (!this.images[i].href)
            this.images[i]=DynAPI.getImage(this.treeImages+"/"+this.imagePrefix+this.images[i].src,this.images[i].w,this.images[i].h)
    }
    if (this.root) this.root.fixImages();
}

/**
  * Finds an item in the tree with the specified string value.
  */
Tree.prototype.findItem=function(item) {
    for (var i=0;i<this.masterIndex.length;i++)
        if (this.masterIndex[i].text==item) return this.masterIndex[i];
    return null;
}

/** Returns the path to this node. 
  * A tree path is an array of indexes of each node's child, 
  * starting from the root node. 
  */
Tree.prototype.getPath=function(node) {
    var c=node;
    var path=[];
    while(c!=this.root) {
        path[path.length]=c.index;
        c=parent;
    }
    return path.reverse();
}

Tree.prototype.setSelected=function(node) {
    var i=0;
    var mi=node;
    if (!node.selected) return node.setSelected(true); // if this method was called outside of TreeNode.setSelected, do the call.
    if (typeof(node)!="number" && node.constructor==TreeNode) mi=node.masterIndex;
    if (this.selectedItem.length>0)
    while (i<this.selectedItem.length)
        if (mi!=this.selectedItem[i]) this.masterIndex[this.selectedItem[i++]].setSelected(false); // force de-select;
    this.selectedItem=[mi];
}

Tree.prototype.addSelected=function(node) {
    var mi=node;
    var i=0;
    if (typeof(node)!="number" && node.constructor==TreeNode) mi=node.masterIndex;
    while (i<this.selectedItem.length && mi!=this.selectedItem[i])
        i++
    if (i<= this.selectedItem.length)
    this.selectedItem[this.selectedItem.length]=mi;
}

Tree.prototype.removeSelected=function(node) {
    var i=0;
    var mi=node;
    if (typeof(node)!="number" && node.constructor==TreeNode) mi=node.masterIndex;
    while (i<this.selectedItem.length && mi!=this.selectedItem[i])
        i++
    if (i<= this.selectedItem.length)
        this.selectedItem=(new Array()).concat(this.selectedItem.slice(0,i),this.selectedItem.slice(i+1));
}

Tree.prototype.getSelectedIndexes=function() {
    return this.selectedItem;
}

Tree.prototype.getSelectedNodes=function() {
    var selList=[];
    for (var i=0;i<this.selectedItem.length;i++)
        selList[i]=this.masterIndex[this.selectedItem[i]];
    return selList;
}

function TreeNode(text,value,renderObj) {
	this.value = value;
	this.text = text;
    this.level=0; // The nesting level of this node. Set when added to a parent node.
    if (renderObj) { // set default render object if one not specified.
        this.renderObj=renderObj;
        renderObj.getComponent().setParentComponent(this);
    }
    this.treeLine=null; // line that links this node to it's sibling.
    this.tree=null; // not yet a member of a tree.

    this.parent=null;
    this.index=-1;
    this.masterIndex=-1;
    
	this.component = new DynLayer(); // the node component.
	this.component.onCreate=function() {
	    var o=this.target;
        if (o.tree==null) {
            return;
        }
	    o.setWidth(o.treeConnImg.getWidth()+o.renderObj.getWidth());
	    if (o.masterIndex!=0)
	        if (o.tree.hideRoot && o.parent.masterIndex==0)
        	    o.moveTo(0,o.treeConnImg.getHeight()*(o.index+1));
        	else
        	    o.moveTo(o.treeConnImg.defaultImage.w,o.treeConnImg.getHeight()*(o.index+1));
    	o.setSize(o.renderObj.getWidth()+o.treeConnImg.getWidth(),o.treeConnImg.getHeight());
    	o.renderObj.setX(o.treeConnImg.getWidth());
	}
	    
	this.component.setParentComponent(this);
	this.component.setBgColor(this.style.bgColor);
	this.component.setBgImage(this.style.bgImage);
	this.component.target=this;

	this.testEvent=new EventListener(this);
	
	this.testEvent.onselect=function(e) {
        e.target.expand();
	}

	this.testEvent.ondeselect=function(e) {
        e.target.close();
    }
    	
    this.children=[]; // empty array.
    this.expandWidth=0; // the max width
    this.expandHeight=0;
    this.mousedown=false;
    this.doHover=false; // enable onHover Events.

    this.rEvents = new EventListener(this);
    
    // Define the events to accept for the render object:
    
    this.rEvents.onmousedown = function (e) {
    	var o = e.getTarget();
    	// Check if the user clicked on a render obj layer.
    	o.mousedown=true;
    }
    
    this.rEvents.onmouseup = function (e) {
    	var o = e.getTarget();
    	// Check if the user clicked on a render obj layer.
    	//o.style.bgColor=o.component.getBgColor();
    	if (o.mousedown) {
    	    o.setSelected(!(o.selected));
    	}
    	o.mousedown=false;
    }
    
    this.rEvents.onmouseover = function (e) {
    	var o = e.getTarget();
        o.style.bgColor=o.getBgColor();
        o.style.textColor=o.getTextColor();
    	if (!o.selected) {
    	    o.renderObj.component.setBgColor(o.style.bgColorRoll);
    	    o.setTextColor(o.style.textColorRoll); // this redirects to the render obj ONLY if it is a DefaultRenderObj
    	}
//    	if (o.showFullNameOnHover) {
//    		setTimeout(o.toString()+'.invokeEvent("hover");',o.hoverPeriod);
//    		o.doHover=true;
//        }
    	o.isMouseOver = true;
    }
    
    this.rEvents.onmouseout = function (e) {
    	var o = e.getTarget();
    	o.mousedown=false;
    	if (!o.selected) {
    	    o.renderObj.component.setBgColor(o.style.bgColor);
    	    o.setTextColor(o.style.textColor);
    	}
    	o.doHover=false;
    	o.isMouseOver = false;
    }
}


WidgetInterface.inherit(TreeNode);

TreeNode.prototype.deselectable = true
TreeNode.prototype.selected = false
TreeNode.prototype.expanded = false
// Copy the master tree style.
TreeNode.prototype.style = Tree.prototype.style

/**
  * Called by events, could also be called manual. This does NOT mean expand.
  * param b Whether to select or deslect.
  */
TreeNode.prototype.setSelected = function(b) { 
    if(this.selected!=b) {
       this.selected=b
    	if (b) {
    	    //this.style.bgColor=this.getBgColor();
    		this.setBgColor(this.style.bgColorSelect);
    		this.setTextColor(this.style.textColorSelect);
    		if (!this.tree.multiSelect)
                this.tree.setSelected(this);                    
            else 
                this.tree.addSelected(this);
    		this.invokeEvent("select");
    		this.tree.lastSelected=this;
    		this.tree.invokeEvent("select");
    		this.tree.lastSelected=null;
    	}
    	else if (this.deselectable) {
    		this.setBgColor(this.isMouseOver?this.style.bgColorRoll:this.style.bgColor);
    		this.setTextColor(this.isMouseOver?this.style.textColorRoll:this.style.textColor);
    		this.tree.removeSelected(this);
    		this.invokeEvent("deselect");
    		this.tree.lastSelected=this;
    		this.tree.invokeEvent("deselect");
    		this.tree.lastSelected=null;
    	}
	}
	return this;
}

TreeNode.prototype.calcMaxWidth=function() {
    with (this) {
        var maxWidth=renderObj.getWidth()+treeConnImg.getWidth();
        if (!hasChildren()) return maxWidth;
        if (!expanded) return maxWidth;
        var nodeWidth=0; //add indent of nodes.
        for (var i=0;i<children.length;i++) {
            nodeWidth=children[i].calcMaxWidth();
            if (nodeWidth>maxWidth) maxWidth=nodeWidth;
        }
        return children[0].getX()+maxWidth;
    }
}

TreeNode.prototype.calcMaxHeight=function() {
    with (this) {
        var height=treeConnImg.getHeight();
        if (hasChildren() && expanded)
            for (var i=0;i<children.length;i++)
                height+=children[i].calcMaxHeight();
        return height;
    }
}

TreeNode.prototype.reCalcSize=function(xDiff,yDiff) {
    with(this) {
        setSize(getWidth()+xDiff,getHeight()+yDiff);
        if (expanded && this.hasSiblings() && !this.isLast() && this.tree.showJoinLines) {
            this.treeLine=tree.getLine();
            addChild(treeLine);
            treeLine.moveTo(0,treeConnImg.getHeight());
            treeLine.setSize(treeLine.getWidth(),getHeight()-treeConnImg.getHeight());
        } else if (!expanded && treeLine!=null) {
            tree.releaseLine(treeLine);
            treeLine=null;
        }
        
        // Now adjust this node's siblings after this node.
        if (this.masterIndex!=0 && parent.children.length-1>index) {
            var total=parent.children.length;
            for(var i=index+1;i<total;i++) {
                parent.children[i].setY(parent.children[i].getY()+yDiff);
            }
        }
        if(this.masterIndex>0) parent.reCalcSize(xDiff,yDiff);
    }
}

/** Expand the tree node, revealing all children. */
TreeNode.prototype.expand=function() {
    with(this) {
        if (!hasChildren() || expanded) return;
        this.expanded=true;
        var xDiff=calcMaxWidth()-getWidth();
        var yDiff=calcMaxHeight()-getHeight();
        reCalcSize(xDiff,yDiff);
        // For some reason, when the numbers are exact, the entire DIV is blacked out in IE5
        tree.setSize(tree.getWidth()+xDiff,tree.getHeight()+yDiff);//+(masterIndex==0?1:0));
    }
}

/** Close the tree node, hiding all children. */
TreeNode.prototype.close=function() {
    with(this) {
        if (!expanded) return;
        expanded=false
        var xDiff=calcMaxWidth()-getWidth();
        var yDiff=calcMaxHeight()-getHeight();
        reCalcSize(xDiff,yDiff);
        tree.setSize(tree.getWidth()+xDiff,(tree.getHeight()+yDiff));//-(masterIndex==0?1:0));
    }
    this.expanded=false;
}

/** set default width */
//TreeNode.prototype.setWidth = function(w) {this.label.setWidth(w)}
/** set default height */
//TreeNode.prototype.setHeight = function(h) {this.label.setHeight(h)}
/** set default width and height*/
//TreeNode.prototype.setSize = function(w,h) {this.label.setSize(w,h)}

TreeNode.prototype.setBgColor=function(color) {
    this.renderObj.setBgColor(color);
}

TreeNode.prototype.getBgColor=function() {
    return this.renderObj.getBgColor();
}

TreeNode.prototype.setTextColor=function(color) {
    this.textColor=color;
    if (this.renderObj&& this.renderObj.constructor==DefaultRenderObj)
        this.renderObj.setTextColor(color);
}

TreeNode.prototype.getTextColor=function() {
    if (this.renderObj&& this.renderObj.constructor==DefaultRenderObj) 
        return this.renderObj.getTextColor();
}

/** by default tree nodes use labels like things as their render object. This is a shortcut to the set text of that label */
TreeNode.prototype.setText = function(text) {
    if (this.renderObj.getClass()==DefaultRenderObj)
        this.renderObj.setText(text);
}
/** by default tree nodes use labels as their render object. This is a shortcut to the get text of that label */
TreeNode.prototype.getText = function(text) {
    if (this.renderObj.getClass()==DefaultRenderObj)
        return this.renderObj.text
    return null;
}

/**
  * Inserts a node within this node.
  */
TreeNode.prototype.insert = function(node,position) {
	if (node.getClass()==TreeNode && node.index==-1) {
		if (typeof(position)!='undefined' && position<this.children.length)
		{
            this.children=(new Array()).concat(this.children.slice(0,position),node,this.children.slice(position));
		    for (var i=this.children.length-1;i>position;i--) {
		        this.children[i].index=i;
		    }
		} else position=this.children.length;
	    node.index=position;
	    this.children[position]=node;
		node.parent = this;
		
        this.addChild(node); // add component of node.
        node.setRelations();
        if (this.component.created) {
            //if (!this.expanded) return;
    	    node.setWidth(node.treeConnImg.getWidth()+node.renderObj.getWidth());
    	    var yOff=this.getY();
        	node.setSize(node.renderObj.getWidth()+node.treeConnImg.getWidth(),node.treeConnImg.getHeight());
    	    if (node.index>0) {
    	        yOff=this.children[node.index-1].calcMaxHeight()+this.children[node.index-1].getY();
    	    }
    	    if (node.masterIndex!=0) {
    	        if (node.tree.hideRoot && node.parent.masterIndex==0)
            	    node.moveTo(0,node.treeConnImg.getHeight()*(node.index+1));
            	else
            	    node.moveTo(node.treeConnImg.defaultImage.w,node.treeConnImg.getHeight()*(node.index+1));
            }
            var j=0;
        	for(var i=node.index;i<this.children.length;i++,j++)
        	    this.children[i].moveTo(this.children[i].treeConnImg.defaultImage.w,(this.children[i].treeConnImg.getHeight()*j)+yOff);
        	node.renderObj.setX(node.treeConnImg.getWidth());
            var xDiff=this.calcMaxWidth()-this.getWidth();
            var yDiff=this.calcMaxHeight()-this.getHeight();
            this.reCalcSize(xDiff,yDiff);
            this.tree.setSize(this.tree.getWidth()+xDiff,this.tree.getHeight()+yDiff);//+(masterIndex==0?1:0));
        }
    } else
        if (node.index!=-1) {
//            alert("Adding node which already exists in tree:"+node.text);
            return null;
        }
    return node;
}

/** Sets up relations with the information in the parent node. 
  * This is important due to the inability to display the icons 
  * without the source locations which is stored in the tree.
  */
TreeNode.prototype.setRelations=function() {
    if (this.parent!=null) { // gotta add the node before it can be related.
        this.tree=this.parent.tree;
        if (!this.treeConnImg) {
            this.treeConnImg=new TreeButtonImage(this);
    	    this.addChild(this.treeConnImg); // the image used to expand/contract the tree.
            this.treeConnImg.addEventListener(this.testEvent);
    	}
        if (!this.renderObj) { // set default render object if one not specified.
            this.renderObj=new DefaultRenderObj(this.text);
            this.renderObj.setTextColor(this.style.textColor);
        	this.renderObj.addEventListener(this.rEvents)
    	    this.addChild(this.renderObj); // The component to render as the visual node. beside the activator and icon.
        }
        this.tree.addNode(this,1)
        if (this.children.length<2) 
        {
            this.fixImages();
            for (var i in this.children)
            // following used instead of fixImages in case adding node with children that have not been added to the tree.
                this.children[i].setRelations(); 
        }
    }
}

TreeNode.prototype.setIcons=function(dImage,sImage) {
    this.treeConnImg.setIcons(DynAPI.getImage(dImage,this.treeConnImage.icon.w,this.treeConnImage.icon.h),DynAPI.getImage(sImage,this.treeConnImage.sIcon.w,this.treeConnImage.sIcon.h));
}

TreeNode.prototype.fixImages=function(rec) {
with(this) {
    if (this.masterIndex==0) {
        treeConnImg.setIcons(tree.images.parentNodeClose,tree.images.parentNodeOpen);
        if (tree.hideRoot) {
            treeConnImg.setImages(tree.images.blankClose,tree.images.blankOpen);
            treeConnImg.selectable=false;
        } else {
            treeConnImg.setImages(tree.images.topNodeClose,tree.images.topNodeOpen);
            treeConnImg.selectable=true;
        }
        return;
    }
    if ((!rec) && this.index<2 && parent.masterIndex!=0)
        parent.fixImages(true); // avoid recursion
    if (hasChildren()) { // use folder and clickable +/-
        treeConnImg.selectable=true;
        treeConnImg.setIcons(tree.images.parentNodeClose,tree.images.parentNodeOpen);
        if (isLast()) {
            treeConnImg.setImages(tree.images.lastNodeClose,tree.images.lastNodeOpen);
            if ((!rec) && this.index>0) 
                parent.children[this.index-1].fixImages(true); // avoid recursion
        } else
            treeConnImg.setImages(tree.images.nodeClose,tree.images.nodeOpen);
    } else {
        treeConnImg.selectable=false;
        treeConnImg.setIcons(tree.images.emptyNodeClose,tree.images.emptyNodeOpen);
        if (isLast()) {
            treeConnImg.setImages(tree.images.lastNode,tree.images.lastNode);
            if ((!rec) && this.index>0) parent.children[this.index-1].fixImages(true); // avoid recursion
        } else
            treeConnImg.setImages(tree.images.node,tree.images.node);
    }
}
}

TreeNode.prototype.append = function(node) {
    this.insert(node);
}

/**
  * Removes a child node from this node at the specified position
  * If no parameters, it is exactly as if removeFromParent was called.
  *
  * param position The child node to remove, or the index of the child node.
  */
  // Remove child and dis-associate from tree.
TreeNode.prototype.remove = function(position) {
    with(this) {
        if (typeof(position)=="object") position=position.getIndex();
        if (!position) {
            return parent.remove(getIndex());
        }
        var node=children[position];
        if (node.masterIndex==0) return; // cannot remove root node.
        if (node.expanded) node.close();
        if (node.selected) node.setSelected(false); // deselect node if it's selected.
        node.parent=null;
        node.index=-1;
        yOff=calcMaxHeight()+getY();

    	for(var i=children.length-1;i>position;i--) {
    	    children[i].setY(children[i-1].getY());
    	    children[i].index=i-1;
    	}
        children[i].moveTo(children[position].treeConnImg.defaultImage.w,node.getY());
        children=(new Array()).concat(children.slice(0,position),children.slice(position+1));
        tree.remove(node);
        // what if we want to keep the relations with the node's children? The following fixes this situation.
        // Use removeAll to completely remove all nodes from the tree and their parent.
        if (node.hasChildren()) node.disAssociate(); // disassociate all children first. 
        node.component.removeFromParent();
        var xDiff=calcMaxWidth()-getWidth();
        var yDiff=calcMaxHeight()-getHeight();
        reCalcSize(xDiff,yDiff);
        tree.setSize(tree.getWidth()+xDiff,tree.getHeight()+yDiff);//+(masterIndex==0?1:0));
        return node; // return the node removed.
    }
//    this.children[i]=null;
}

/** Disassociates this nodes children from the tree, but does not remove it from this node. */
TreeNode.prototype.disAssocate= function() {
    if (this.tree!=null) return; // this node must be disassociated first.
    for (var i=0;i<this.children.length;i++) {
        if (this.children[i].selected) this.children[i].setSelected(false);
        this.tree.remove(this.children[i]);
        if (this.children[i].hasChildren()) this.children[i].disAssocate();
    }
}

/**
  * Removes all child nodes from this node.
  */
TreeNode.prototype.removeAll = function() {
    for(var i=this.getChildCount();i>0;i--)
        this.remove(i);
}

TreeNode.prototype.removeFromParent = function() {
    this.parent.remove(this.getIndex());
}

/**
  * Setting parent also removes this node from it's existing parent.
  */
TreeNode.prototype.setParent = function(parentNode) {
    if (this.masterIndex==0) return;
    if (this.parent!=null)
        this.parent.remove(this); // Remove from existing parent.
    parentNode.append(this);  // Add to new parent.
}

TreeNode.prototype.getParent = function() {
    return this.parent;
}

TreeNode.prototype.getChildren = function() {
    return this.children;
}

TreeNode.prototype.getChildCount = function() {
    return this.children.length;
}

TreeNode.prototype.isLast=function() {
    if (!this.hasSiblings()) return true;
    if (this.parent.children.length-1==this.index) return true;
    return false;
}

/* returns true if this TreeNode has others within it's parent. */
TreeNode.prototype.hasSiblings=function() {
    if (this.parent==null || this.masterIndex==0) return false;
    if (this.parent.children.length>1) return true;
    return false;
}

TreeNode.prototype.hasChildren = function() {
    if (this.children.length>0) return true;
    return false;
}

TreeNode.prototype.getIndex = function(node) {
    return this.index;
}

TreeNode.prototype.isLeaf = function() {
    return !this.hasChildren();
}

TreeNode.prototype.setText = function(text) {
    this.text=text;
}

TreeNode.prototype.getText = function() {
    return this.text;
}

/**
  * The render object is the object which is designated to be rendered in place of the tree node.
  * This must be of sub-type widget. DynLayer also inherits widget.
  */
TreeNode.prototype.setRenderObject = function(rObj) {
    if (rObj.getComponent().getClass()!=DynLayer) return null;
    this.renderObj=rObj;
	this.renderObj.setParentComponent(this);
}

/**
  * Retrieves the object designated to be rendered in this tree node's place.
  */
TreeNode.prototype.getRenderObject = function() {
    return this.renderObj;
}

/** Used to connect 2 tree nodes with a vertical line, if it is > 1 node away */
function TreeLine(treeNode) {
	this.component = new DynLayer()
	this.component.setParentComponent(this)
	this.image=treeNode.tree.images.line.src;
	this.setBgImage(this.image);
	this.component.setSize(treeNode.tree.images.line.w,treeNode.tree.images.line.h);
	//this.setBgColor("blue");
}
WidgetInterface.inherit(TreeLine);

function TreeButtonImage(treeNode) {
    this.component = new DynLayer();
    this.component.setParentComponent(this);
    this.defaultImage=treeNode.tree.images.topNodeClose;
    this.selectedImage=treeNode.tree.images.topNodeOpen;
    this.icon=treeNode.tree.images.emptyNodeClose;
    this.sIcon=treeNode.tree.images.topNodeOpen;
    this.selected=false;
    this.selectable=false;
    this.mousedown=false;
    this.parent=treeNode;
    this.component.target=this;
    
    this.component.onCreate=function() {
        this.target.updateHTML();
    }

    this.events = new EventListener(this)

    this.events.onmousedown = function (e) {
        var o=e.getTarget();
        if (e.getX()>o.defaultImage.w) return;
        o.mousedown=true;
        e.setBubble(false);
    }

    this.events.onmouseup = function (e) {
        var o=e.getTarget();
        e.setBubble(false);
        if (e.getX()>o.defaultImage.w) return;
        if (o.mousedown)
            o.setSelected(!o.selected);
        e.setBubble(true);
    }

    this.events.onmouseout = function (e) {
        e.getTarget().mousedown=false;
        //e.setBubble(false);
    }
    this.component.addEventListener(this.events);
}

WidgetInterface.inherit(TreeButtonImage);

TreeButtonImage.prototype.setImages = function(dImage,sImage) {
    with(this) {
    	defaultImage = dImage;
    	selectedImage = sImage;
    	if (component.created) change((selected?selectedImage:defaultImage));
    }
}

TreeButtonImage.prototype.setIcons = function(dIcon,slIcon) {
    this.icon = dIcon;
    this.sIcon = slIcon;
    with(this) {
        if (component.created) setIcon((selected?sIcon:icon));
    }
}

/** For changing the image type on this tree node.. */
TreeButtonImage.prototype.changeDefaultImage=function(imageSrc) {
    with(this) {
        defaultImage = imageSrc;
        if (!selected && component.created) change(defaultImage);
    }
}

TreeButtonImage.prototype.updateHTML=function() {
    with(this) {
        component.setHTML('<table cellspacing=0 cellpadding=0 border=0 align=left><tr><td width="'+
            (!selected?(defaultImage.w+icon.w):
                       (selectedImage.w+sIcon.w))+
            '"><img name="'+component.id+'Image" src="'+
            (!selected?(defaultImage.src+'" width='+defaultImage.w+' height='+defaultImage.h):
                      (selectedImage.src+'" width='+selectedImage.w+' height='+selectedImage.h))+
            '></td><td><img name="'+component.id+'Icon" src="'+
            (!selected?(icon.src+'" width='+icon.w+' height='+icon.h+'></td></tr></table>'):
                       (sIcon.src+'" width='+sIcon.w+' height='+sIcon.h))+
            '></td></tr></table>');
        component.setSize(defaultImage.w+icon.w,icon.h);
    }
}

TreeButtonImage.prototype.changeSelectedImage=function(imageSrc) {
	this.selectedImage = imageSrc;
	if (this.selected && this.component.created) this.change(this.selectedImage);
}

TreeButtonImage.prototype.setSelected = function(b) {
    with(this) {
        if (!selectable) return;
        selected=b;
    	if (selected) {
    		change(selectedImage);
    		setIcon(sIcon);
    		invokeEvent("select");
    	}
    	else {
    		change(defaultImage);
    		setIcon(icon);
    		invokeEvent("deselect");
    	}
    }
}

/** Change the icon shown in the node. **/
TreeButtonImage.prototype.setIcon = function(img) {
    if (typeof(image)=="string") img=DynAPI.getImage(img);
//    this.icon=img;
    if (img && this.parent.component.created && this.parent.expanded) this.component.doc.images[this.component.id+"Icon"].src = img.src;
}

/** Temporarly change the image that is visible **/
TreeButtonImage.prototype.change = function(img) {
    if (typeof(image)=="string") img=DynAPI.getImage(img);
	if (img) this.component.doc.images[this.component.id+"Image"].src = img.src;
}

function DefaultRenderObj(text,textColor) {
    this.label=new Label(text,false);
    this.textColot=textColor;
    this.component=this.label.component;
    this.label.addCover();

    this.events = new EventListener(this)

    this.events.onmousedown = function (e) {
        e.setBubble(false);
        e.getTarget().invokeEvent("mousedown");
    }

    this.events.onmouseup = function (e) {
        e.setBubble(false);
        e.getTarget().invokeEvent("mouseup");
    }

    this.events.onmouseout = function (e) {
        e.setBubble(false);
        e.getTarget().mousedown=false;
        e.getTarget().invokeEvent("mouseout");
    }

    this.events.onmouseover = function (e) {
        e.setBubble(false);
        e.getTarget().mousedown=false;
        e.getTarget().invokeEvent("mouseover");
    }
    this.label.coverlyr.addEventListener(this.events);
}

DefaultRenderObj.prototype.setTextColor=function(color) {
    this.textColor=color;
    this.label.setTextColor(color);
}

DefaultRenderObj.prototype.getTextColor=function() {
    return this.textColor;
}

WidgetInterface.inherit(DefaultRenderObj);
