// *******************************************
// **
// ** Author: Brandon E. Myers
// **   Date: I don't know. Umm.. today? 12/01/1997
// ** E-Mail: Brandon_Myers@hotmail.com
// **
// *******************************************
// **
// ** Permission to use this Script is granted provided that
// ** you leave this file fully intact and this disclaimer
// ** is included on any and all pages which use any of the
// ** functions below. Also send an E-mail to me to let me know
// ** that my time was not wasted, and maybe I'll have an updated
// ** version with a lot of nifty new features.
// **
// *******************************************
// **
// ** Requires debug.js: (Only if debuging) Debugging tools to be included before this script.
// **
// *******************************************
// ** 
// ** System Event object definitions. 
// **
// ** These functions allow multiple dynamic event handeling
// ** AND dynamic prioritizing of events.
// ** 
// *******************************************
// ** 
// ** Last updated 5/1/2000
// **   Summary of changes: (Brandon E. Myers)
// **     Full overhaul of structure to conform to manipulating object prototypes to add object methods.
// **     Renamed methods to be more intuitive. Changed name of eventObj to be EventHandler.
// ** 
// *******************************************

function EventHandler(Owner) {
	debug.log('Construct: eventObj<BR>Owner: '+(typeof(Owner)=="undefined"?"None":Owner.name)+'<BR>',debug.CONSTRUCT);
	if (typeof(Owner)!="undefined") this.parent=Owner;
	this.typeOf='EventHandler';
	this.enabled=true;
}

eventObj=EventHandler; // for backward compatability

eventObj.prototype.addType=function(eventDesc) {
    if(eventDesc==null || (typeof(eventDesc)!="object" && typeof(eventDesc)!="function")) {
	    alert("eventObj.createType: Cannot get string object from argument 1.\nReturning null.");
	    return null;
	}
	this[eventDesc.type]=eventDesc;
	return eventDesc; // return the event descriptor.
}

eventObj.prototype.createType=function(Type) {
	debug.log('Method: createType<BR>Type: '+Type+'<BR>',debug.CONSTRUCT);
	if (typeof(Type)!="string") {
	    alert("eventObj.createType: Cannot get string object from argument 1.\nReturning null.");
	    return null;
	}
	tmpObj=new Object(); // Define object for custom user events.
	tmpObj.order=new Array(0);
	tmpObj.exec=new Array(0);
	tmpObj.name=new Object();
	tmpObj.NoExec=false;
	tmpObj.nameIndex=new Array(0);
	tmpObj.enabled=new Array(0);
	tmpObj.abortExec=false;
	tmpObj.abort=abortEvent;
	tmpObj.type=Type;
	tmpObj.parent=this; // reference to the top level event object.
	this.addType(tmpObj); // add to main event object.
	return tmpObj; // return the new event object for quick reference by other code
}

eventObj.prototype.addEvent=function(type,event) {
	if (typeof(event)=="string") {
	    event=new jsEvent(event)
	}
	var theindex=this.order.length;
	this[type].addEvent(event); // It's the descriptors responsbility to maintain the events..
}

eventObj.prototype.enable = function (Type,Name) {
	var thisobj=this.parent;
//	debug.log('enableEvent '+this.parent.typeOf+': '+Type,DEBUG_METHOD);
	if (typeof(Name)=='string') { // If the event name was specified.
		this.enabled[this[Type].name[Name].index]=true;  // Enable Specific Event.
	} else {
//		for (var i in this.enabled) { // Get each event.
//			this.enabled[i]=true; // Enable ALL envents of type Type;
        this[Type].NoExec=true;
		}
	}
}

eventObj.prototype.disable = function (Type,Name) {
	var thisobj=this.parent;
	// debug.log('disableEvent '+thisobj.typeOf+': '+Type,DEBUG_METHOD);
	if (typeof(Name)=='string') { // If the event name was specified.
		this.enabled[this[Name].index]=false;  // Enable Specific Event.
	} else {
//		for (var i in this.enabled) { // Get each event.
//			this.enabled[i]=false; // Enable ALL events of type Type;
        this[Type].NoExec=false;
		}
	}
}

function eventDescriptor(Type) { // this object describes event types.
	debug.log('Construct: eventDescriptor<BR>Type: '+Type+'<BR>',debug.CONSTRUCT);
	this.order=new Array(0); // Array for ordering execution of multiple events.
	this.exec=new Array(0);  // Array of code to execute.
	this.name=new Object();  // Hash of event names to event objects.
	this.NoExec=false;       // Whether to not execute events of this type.
	this.nameIndex=new Array(0); // Array index of number to name.
	this.isFunction=new Array(0);// Array of whether the action item is an object or a string to be evaluated.
	this.enabled=new Array(0); // Whether each event is enabled or not.
	this.abortExec=false;    // This switch is set to false before each event execution. 
	                         // If true after event execution, further events of this type are not executed.
	this.abort=abortEvent;   // if set, this event was aborted and an error occured.
	this.type=Type;          // Specifies the event type.
}

/**
  * Trigger an event within this descriptor.
  *
  * param Name (Optional) The name of the event to execute. If no name is given, 
  *            all events within this descriptor are executed in order.
  * param varObj (Optional) An object to send to the event. If the event is a function,
  *              it is passed as a parameter. Otherwise, it is accessable 
  *              from the evaluated code using the name varObj. 
  *              If a name is not specified, this is the only arguement.
  */
eventDescriptor.prototype.trigger=function(Name, varObj) {
	varObj=(typeof(varObj)=='object'?varObj:new Object());
    return ((typeof(Name)=='undefined')||(Name=='')?this.parent.trigger(this.type,'',varObj):this.parent.trigger(this.type,Name,varObj))
//	doEvent(this.type,Name,this.parent);
}

/**
  * Create a new event using the descriptor object.
  */
eventDescriptor.prototype.newEvent=(Name,action,priority) { //create an event.
    // must specify an action...
    if (typeof(action)=='undefined' || typeof(action)=='number' || typeof(action)=='integer') return null;
	if (typeof(priority)=="undefined") priority=10;
	tmpObj=new jsEvent(Name,action,priority);
	tmpObj.index=theindex;
	var theindex=this.order.length;
	this.order[theindex]=priority;
	this.name[Name]=tmpObj;
	this.nameIndex[theindex]=Name;
//	this.exec[theindex]=action;
//	this.enabled[theindex]=true;
//	this.isFunction[theindex]=theindex;
	this[Name]=event;
}

/**
  * This is the actual event which holds code information.
  */
function jsEvent(Name,action,priority) {
//	alert("NewEvent:"+Type+":"+Name+":"+action);
    if (typeof(action)=='undefined') return null;
	if (typeof(priority)=="undefined") priority=10;
	this.parent=this;
	this.name=Name;
	this.index=theindex;
	this.order=priority;
	this.exec=action;
	this.enabled=true;
	if (typeof(action)=='function' || typeof(action)=='object') 
	    this.isFunction=true;
	else 
	    this.isFunction=false;
	this.type=Type;
	this.typeOf='event';
}

jsEvent.prototype.trigger=function(varObj) {
	varObj=(typeof(varObj)=='undefined'?(new Object()):varObj);
    return this.parent.parent.trigger(this.type,this.name,varObj); // user parent method to execute.
}


function _doEvent(Type,Name,varObj) {
	if (this.NoExec) return false;
	var thisevent=this;
	var thisobj=thisevent.parent; // Assign Reference to the object for which the event happened.
	var retval=true;
	var tmpretval=true;
	var currentevent=thisevent;
	debug.log('Event::('+thisobj.typeOf+')'+thisobj.name+'.'+Type,debug.EVENT);
	if (typeof(Name)=='object') {
		varObj=Name;
		Name='';
	} else varObj=(typeof(varObj)=='object'?varObj:new Object());
//	debug.log('Event Trigger:<BR>&nbsp;&nbsp;'+thisobj.typeOf+'.'+Type+(typeof(Name)!="undefined"?Name:''),debug.METHOD);
	if (typeof(Name)=='string' && Name!='') { // If the event name was specified.
		if (currentevent.enabled[currentevent.name[Name].index]==true) {
			debug.log('<FONT COLOR=RED>&nbsp;&nbsp;Event: '+Name+'<BR></FONT>',debug.EVENT);
			thisevent.NoExec=true;
			retval=eval(currentevent.exec[currentevent.name[Name].index]); // Run specified event.
			thisevent.NoExec=false;
			if (currentevent.abortExec) { currentevent.abortExec=false; }
			return retval;
		}
	    return false; // No event fired, or the event does not exist.
	} else {
		var tempArray=copyArray(currentevent.order);
		tempArray=tempArray.sort();
		var tempIndex=0;
		var prevPriority=999;
		for (var j in tempArray) { // Get each event in priority order.
			if (j=='length') continue; // Prevent javascript error.
			if (prevPriority==tempArray[j]) continue;  // Prevent running events with the same priority twice.
			prevPriority=tempArray[j];
			for (var i in currentevent.name) {
			    if (i=='length') continue;
				tempIndex=currentevent[i].index;
				if (tempArray[j] == currentevent.order[tempIndex]) // if order correct then...
				if (currentevent.enabled[tempIndex]==true) {
        			debug.log('<FONT COLOR=RED>&nbsp;&nbsp;Event: '+i+'</FONT>',debug.EVENT);
					thisevent.NoExec=true; // Disable processing of this type of event.
					tmpretval=eval(currentevent.exec[tempIndex]); // Run the event.
					thisevent.NoExec=false; // Enable processing of this type of event.
					tmpretval=(typeof(tmpretval)!='boolean'?true:tmpretval);  // make sure tmpretval is a boolean value.
					retval=(retval & tmpretval); // If at anytime false is returned... return false at the end of all events.
					debug.log("<FONT COLOR=GREEN>&nbsp;&nbsp;Returned: "+retval+"</FONT><BR>",debug.EVENT);
					if (currentevent.abortExec) { currentevent.abortExec=false; return retval; }
				}
			}
		}
	}
	return retval;
}

function abortEvent() {
	this.abortExec=true;
	return false;
}

//var tdebug=typeof(debug);
if (typeof(debug)!='object') {
    debug=new Object();
    debug.log=function (){return;}
}

	this.trigger=doEvent;
	this.enable=enableEvent;
	this.disable=disableEvent;
	this.newEventType=newEventType;
	this.newEvent=addNewEvent; // Function to add an event to the event list.
