<?

/*

PHeadlines Class 0.11 (July 2, 2000)
a news headline grabbing engine which caches the links in a MySQL database
http://www.dansteinman.com/pheadlines/

Copyright (C) 1999 Dan Steinman (dan@dansteinman.com)
This is free software under the terms of the GNU General Public License

*/

class PHeadlines {
	// how many minutes to cache the links for
	var $cache_minutes = 60;

	// number of links to display
	var $link_limit = 10;

	// dns name of the http proxy server, leave empty if none
	var $proxy = "";

	// port of the http server running on $proxy
	var $proxy_port = "8080";
	
	// List of sites
	var $sites = array(	
		"slashdot" => array("Slashdot","http://slashdot.org","/slashdot.rdf","rdf"),
		"freshmeat" => array("Freshmeat","http://freshmeat.net","/backend/fm.rdf","fm"),
		"linuxtoday" => array("Linux&nbsp;Today","http://linuxtoday.com","/backend/linuxtoday.xml","lt"),
		"wired" => array("Wired&nbsp;News","http://www.wired.com","/news_drop/netcenter/netcenter.rdf","rdf"),
		"gametome" => array("Linux&nbsp;Game&nbsp;Tome","http://happypenguin.org","/html/news.rdf","rdf"),
		"segfault" => array("Segfault","http://segfault.org","/stories.xml","sf"),
		"kdenews" => array("KDE&nbsp;News","http://www.kde.org","/news/kdenews.rdf","rdf"),
		"gnomenews" => array("Gnome&nbsp;News","http://news.gnome.org","/gnome-news/rdf","rdf"),
		"gimpnews" => array("Gimp&nbsp;News","http://www.xach.com/gimp/news/","channel.rdf","rdf-chan"),
		"mozilla" => array("Mozilla","http://www.mozilla.org","/news.rdf","rdf-chan"),
		"mozillazine" => array("MozillaZine","http://www.mozillazine.org","/contents.rdf","rdf"),
		"technocrat" => array("Technocrat","http://www.technocrat.net","/rdf","rdf"),
		"appwatch" => array("AppWatch","http://appwatch.com","/appwatch.rdf","rdf"),
		"linuxweekly" => array("Linux&nbsp;Weekly&nbsp;News","http://lwn.net","/headlines/rss","lwn")
	);

	function PHeadlines($db) {
		$this->db = $db;
		if (!$this->db->tableExists("cache_times")) {
			// create db tables
			$this->db->query("create table cache_times (site VARCHAR(16), time INT UNSIGNED)");
			while(list($name) = each($this->sites)) {
				$this->db->query("create table $name (title VARCHAR(255), link VARCHAR(255))");
				$this->db->query("insert into cache_times VALUES(\"$name\",0)");
			}
		}
	}

	// try to get the links for the site
	function getLinks($which) {
		if (!$this->sites[$which]) { // if no such site
			return false;
		}
		else if ($this->isCached($which)) {
			$links = $this->getLinksDB($which);  // get from db
		}
		else {  // cache has expired
			$links = $this->getLinksSite($which);  // get from site

			if ($links) { // successful
				$this->saveToDB($which,$links);  // cache
			}
			else {  // timed out
				$links = $this->getLinksDB($which);  // try from db
				$this->error_timeout = true;
			}
		}
		return $links;
	}

	// determines if the headlines were cached less than $cache_mintues minutes ago
	function isCached($which) {
		$result = $this->db->query("select time from cache_times where(site='$which')");
		if ($result) {
			$row = $this->db->fetch_array($result);
			$cached_time = $row[0];
			$this->current_time = time();		
			return (($this->current_time-$cached_time)<$this->cache_minutes*60);
		}
		else {
			return false;
		}
	}

	// get the links from the database
	function getLinksDB($which) {
		$result = $this->db->query("select * from $which");

		if (!$result || $this->db->num_rows($result)==0) {  // if none in db
			$links = $this->getLinksSite($which);  // try from site again
			if (!$links) {
				$url = $this->sites[$which][1].$this->sites[$which][2];
				die("<b>error</b>: unable to get links for $which<br><a href=\"$url\">$url</a>");
			}
		}
		
		while($row = $this->db->fetch_array($result)) {
			$links[$row['title']] = $row['link'];
		}
		return $links;
	}

	// get a new set of links from the site
	function getLinksSite($which) {
		// determine the options to properly extract the links
		$type = $this->sites[$which][3];
		$startat = "</image>";  // line to start at
		$linkstr = "link";  // the xml tag around the url
		$exclude = "";  // the title of a link to ignore
		
		if ($type=="rdf-chan") $startat = "</channel>";
		else if ($type=="lt") $linkstr = "url";
		else if ($type=="fm") $exclude = "quick finder";
		else if ($type=="lwn") $exclude = "Linux Weekly News";
		else if ($type=="sf") $startat = "</textinput>";
		
		// get the file that contains the links
		$lines = http_get($this->sites[$which][1].$this->sites[$which][2]);
		if (!$lines) return false;
	
		// determine which line to begin grabbing the links
		for ($i=0;$i<count($lines);$i++) {
			if (ereg($startat,$lines[$i],$regs)) {
				$startnum = $i;
				break;
			}
		}

		// extract the links and assemble into array $links
		$links = array();
		for ($i=$startnum;$i<count($lines);$i++) {
			if (count($links)>=$this->link_limit) break;
			if (ereg("<title>(.*)</title>",$lines[$i],$regs)) {
				if ($regs[1] == $exclude) {$i+=1; break;}
				$title = $regs[1];
				$title = ereg_replace("&amp;apos;","'",$title);
				
			}
			else if (ereg("<$linkstr>(.*)</$linkstr>",$lines[$i],$regs)) {
				$links[$title] = $regs[1];
$startnum = $i;
				break;
			}
		}

		// extract the links and assemble into array $links
		$links = array();
		for ($i=$startnum;$i<count($lines);$i++) {
			if (count($links)>=$this->link_limit) break;
			if (ereg("<title>(.*)</title>",$lines[$i],$regs)) {
				if ($regs[1] == $exclude) {$i+=1; break;}
				$title = $regs[1];
				$title = ereg_replace("&amp;apos;","'",$title);
			}
			else if (ereg("<$linkstr>(.*)</$linkstr>",$lines[$i],$regs)) {
				$links[$title] = $regs[1];
			}
		}
		return $links;
	}

	// get a list of the sites
	function getList() {
		while(list($name,$value) = each($this->sites)) {
			$links[$value[0]] = $name;
		}
		return $links;
	}

	// save the new set of links and update the cache time
	function saveToDB($which,$links) {
		$result = $this->db->query("delete from $which");
		
		// save links
		while (list($title,$link) = each($links)) {
			$this->db->query("insert into $which VALUES(\"$title\",\"$link\")");
		}		
		
		// save cache time
		$this->db->query("delete from cache_times where(site='$which')");
		$this->db->query("insert into cache_times VALUES('$which','$this->current_time')");
	}
}

?>
